#!/usr/bin/python3
# -*- coding: utf-8 -*-
from board import SCL, SDA
import busio
import time
import subprocess
import datetime
from threading import Thread
from oled_text import OledText, SmallLine

import netifaces as ni
ip = ni.ifaddresses('eth0')[ni.AF_INET][0]['addr']

i2c = busio.I2C(SCL, SDA)

f_in_filter = '/sys/class/gpio/gpio9/value' # состояние фильтра
f_in_high = '/sys/class/gpio/gpio8/value' # состояние реле подкачки
f_in_low = '/sys/class/gpio/gpio7/value' # состояние обычного реле

f_in_temp = '/sys/bus/w1/devices/28-0301a279404b/temperature' # путь к термодатчику
f_out_power = '/home/rain/out_power' # данные с измерителя мощности

f_out_filter = '/sys/class/gpio/gpio27/value' # режим работы фильтра авто/ручной
f_out_light = '/sys/class/gpio/gpio25/value' # управление подсветкой
f_out_mode = '/sys/class/gpio/gpio22/value' # переключение между реле
f_out_stop = '/sys/class/gpio/gpio23/value' # управление аварийным остановом

f_in_btn1 = '/sys/class/gpio/gpio11/value' # кнопка насоса
f_in_btn2 = '/sys/class/gpio/gpio17/value' # кнопка фильтра

btn1 = 0 # внутренний таймер кнопки насоса
btn2 = 0 # внутренний таймер кнопки фильтра
freg = 0 # регистр состояния фильтра
makepump = '0' # начальное состояние функции подкачки

# инициализация переменных хранения состояния между циклами
oldpumprelay = ''
oldnormrelay = ''
oldmyfilter = ''

# логгирование состояния входов
worklog = '/home/rain/my.log'
mylog = open(worklog, 'a', buffering=1)

# Create the display, pass its pixel dimensions
oled = OledText(i2c, 128, 64)

oled.layout = {
	1: SmallLine(0, 0, font="FontAwesomeSolid.ttf", size=14), # термометр
	2: SmallLine(20, 0, font="Arimo.ttf", size=14), # значение термометра

	3: SmallLine(80, 0, font="FontAwesomeSolid.ttf", size=14), # лампочка
	4: SmallLine(95, 0, font="Arimo.ttf", size=14), # значение подсветки

	5: SmallLine(0, 17, font="Arimo.ttf", size=12), # фильтр, слово
	6: SmallLine(68, 17, font="Arimo.ttf", size=12), # фильтр режим
	7: SmallLine(105, 17, font="Arimo.ttf", size=12), # фильтр состояние

	8: SmallLine(0, 35, font="Arimo.ttf", size=12), # насос слово
	9: SmallLine(68, 35, font="Arimo.ttf", size=12), #  ваттметр

	10: SmallLine(0, 49, font="Arimo.ttf", size=15), # значение H
	11: SmallLine(68, 49, font="Arimo.ttf", size=15), # значение L
	12: SmallLine(53, 52, font="FontAwesomeSolid.ttf", size=11), # state HIGH
	13: SmallLine(115, 52, font="FontAwesomeSolid.ttf", size=11) # state LOW
}

oled.text('System IP:', 2)
oled.text(ip, 5)
time.sleep(1)
oled.auto_show = False

oled.text('\uf769', 1)
oled.text('\uf0eb', 3)
oled.text('ФИЛЬТР:', 5)
oled.text('НАСОС', 8)

# первая запись в лог. В принципе, можно было заюзать datetime.now()
sttime = datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d %H:%M:%S - ')
mylog.write(sttime + 'System start' + '\n')

# функция снятия данных с измерителя. Задержка на Zubr'e - 3 секунды
def pzemkick():
	time.sleep(5)
	subprocess.run(['/home/rain/bin/phase_3_update.sh'])

# основной цикл
while True:
# метка времени в рамках цикла
	sttime = datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d %H:%M:%S - ')

	in_filter = open(f_in_filter, 'r')
	in_high = open(f_in_high, 'r')
	in_low = open(f_in_low, 'r')
	in_temp = open(f_in_temp, 'r')
	out_fstate = open(f_out_filter, 'r')
	out_light = open(f_out_light, 'r')
	out_mode = open(f_out_mode, 'r')
	out_power = open(f_out_power, 'r')
	out_stop = open(f_out_stop, 'r')

# вычитка температуры, деление на 1000 и округление до 2 знака. Иногда бывало пустое значение, так что через переменную.
# в идеале кинуть вычитку в фон, но пока не знаю, как
	currtemp = in_temp.read(5)
	if currtemp != "":
		oled.text(str(round(int(currtemp)/1000, 1)) + " C", 2)

# выбор стиля отображения выбранного реле давления в зависимости от запрета работы - стрелка/крестик
	if out_stop.read(1) == '1':
		state = '\uf060'
	else:
		state = '\uf00d'

	if out_light.read(1) == '1':
		light = '0'
		oled.text('OFF', 4)
	else:
		light = '1'
		oled.text('ON', 4)

	if out_mode.read(1) == '1':
		relayswitch = 1
		oled.text(state, 13)
		oled.text('', 12)
	else:
		relayswitch = 0
		oled.text(state, 12)
		oled.text('', 13)

# обвязка реле подкачки: запись в лог по смене статуса, запуск измерителя
	if in_high.read(1) == '1':
		pumprelay = '1'
		if pumprelay != oldpumprelay:
			mylog.write(sttime + 'Pump relay pressure is less than 3.5 kg' + '\n')
			dopzem = Thread(target=pzemkick)
			dopzem.start
		oled.text('H: ON', 10)
	else:
		pumprelay = '0'
		if pumprelay != oldpumprelay:
			mylog.write(sttime + 'Pump relay pressure is 4.5+ kg' + '\n')
			dopzem = Thread(target=pzemkick)
			dopzem.start
		if makepump == '1':
		# если была включена однократная подкачка и реле сработало - возвращаемся в нормальный режим
			subprocess.run(['gpio', 'export', '22', 'high'])
			makepump = '0'
		oled.text('H: OFF', 10)
	oldpumprelay = pumprelay

# обвязка обычного реле: запись в лог по смене статуса, запуск измерителя
	if in_low.read(1) == '1':
		normrelay = '1'
		if normrelay != oldnormrelay:
			mylog.write(sttime + 'Normal relay pressure is less than 2 kg' + '\n')
			dopzem = Thread(target=pzemkick)
			dopzem.start
		oled.text('L: ON', 11)
	else:
		normrelay = '0'
		if normrelay != oldnormrelay:
			mylog.write(sttime + 'Normal relay pressure is 4.5+ kg' + '\n')
			dopzem = Thread(target=pzemkick)
			dopzem.start
		oled.text('L: OFF', 11)
	oldnormrelay = normrelay

# вычитка измерителя мощности (точнее, файла-прослойки)
	currpower = out_power.read()
	if currpower != "":
		oled.text(str(round(int(currpower)/1000, 2)) + " KW", 9)

#
	if in_filter.read(1) == '1':
		myfilter = '1'
		if myfilter != oldmyfilter:
			mylog.write(sttime + 'Pool filter is on' + '\n')
		oled.text('ON', 7)
	else:
		myfilter = '0'
		if myfilter != oldmyfilter:
			mylog.write(sttime + 'Pool filter is off' + '\n')
		oled.text('OFF', 7)
	oldmyfilter = myfilter

	if out_fstate.read(1) == '1':
		oled.text('AUTO', 6)
	else:
		oled.text('MAN', 6)

# запуск перерисовки экрана в отдельном треде
	display = Thread(target=oled.show)
	display.start()
#	display.join()
#	oled.show()
#	oled.auto_show = True

# работа с кнопками на панели
	in_btn1 = open(f_in_btn1, 'r')
	in_btn2 = open(f_in_btn2, 'r')

	if in_btn1.read(1) == '1':
		btn1 += 1
		if btn1 > 5:
# на долгое нажатие и при включенном реле подкачки - запускаем подкачку и ставим флаг makepump
			if pumprelay == '1':
				makepump = '1'
				subprocess.run(['gpio', 'export', '22', 'low'])
			btn1 = 0
	else:
		if btn1 > 0 and btn1 < 5:
# на короткое нажатие делаем переключение между обычным реле и реле подкачки
			if relayswitch == 0:
				# normal relay mode
				subprocess.run(['gpio', 'export', '22', 'high'])
			else:
				# pump relay mode
				subprocess.run(['gpio', 'export', '22', 'low'])
		btn1 = 0

	if in_btn2.read(1) == '1':
		btn2 += 1
		if btn2 > 5:
# на длинное нажатие включаем/выключаем подсветку бассейна
			if light == '0':
			# light off
				subprocess.run(['gpio', 'export', '25', 'low'])
			else:
			# light on
				subprocess.run(['gpio', 'export', '25', 'high'])
			btn2 = 0
	else:
		if btn2 > 0 and btn2 < 5:
# на короткое нажатие переключаемся между режимами работы фильтра: авто/выкл/вкл
			if freg == 0:
			# manual off
				subprocess.run(['gpio', 'export', '27', 'low'])
				subprocess.run(['gpio', 'export', '18', 'high'])
			elif freg == 1:
			# manual on
				subprocess.run(['gpio', 'export', '27', 'low'])
				subprocess.run(['gpio', 'export', '18', 'low'])
			else:
			# auto
				subprocess.run(['gpio', 'export', '27', 'high'])
				subprocess.run(['gpio', 'export', '18', 'high'])
			freg += 1
			if freg > 2:
				freg = 0
		btn2 = 0

	in_filter.close()
	in_high.close()
	in_low.close()
	in_temp.close()
	out_fstate.close()
	out_light.close()
	out_mode.close()
	out_power.close()
	out_stop.close()

	in_btn1.close()
	in_btn2.close()

	time.sleep(0.25)
